/*
    Video2P -- video hosting engine for I2P network.
    Copyright (C) 2023 - 2025 Video2P Team

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <stdlib.h>

#include "memory.hpp"

V2PAllocatorException::~V2PAllocatorException() throw()
{
}

const char *
V2PAllocatorException::what() const throw()
{
	return "Bad allocation";
}

void *
V2PAllocator::malloc(size_t size) throw(V2PAllocatorException)
{
	void *result = ::malloc(size);
	if (result == NULL) {
		throw V2PAllocatorException();
	}
	return result;
}

void *
V2PAllocator::calloc(size_t num, size_t size) throw(V2PAllocatorException)
{
	void *result = ::calloc(num, size);
	if (result == NULL) {
		throw V2PAllocatorException();
	}
	return result;
}

void *
V2PAllocator::realloc(void *ptr, size_t size) throw(V2PAllocatorException)
{
	void *result = ::realloc(ptr, size);
	if (result == NULL) {
		throw V2PAllocatorException();
	}
	return result;
}

void
V2PAllocator::free(void *ptr) throw()
{
	::free(ptr);
}

void *
operator new(size_t, void *ptr) throw()
{
	return ptr;
}

void *
operator new[](size_t, void *ptr) throw()
{
	return ptr;
}
