/*
    Video2P -- video hosting engine for I2P network.
    Copyright (C) 2023 - 2025 Video2P Team

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef VIDEO2P_SCOPED_PTR_HPP
#define VIDEO2P_SCOPED_PTR_HPP

#include <stddef.h>

template <class T> class V2PScopedPtr {
private:
	T *m_ptr;

public:
	V2PScopedPtr(T *ptr) throw()
	    : m_ptr(ptr)
	{
	}

private:
	V2PScopedPtr(const V2PScopedPtr &);
	V2PScopedPtr &operator=(const V2PScopedPtr &);

public:
	~V2PScopedPtr() throw() { delete m_ptr; }

	T *moveFrom() throw()
	{
		T *ret = m_ptr;
		m_ptr = NULL;
		return ret;
	}

	T *get() const throw() { return m_ptr; }

	T &operator*() const throw() { return *m_ptr; }

	T *operator->() const throw() { return m_ptr; }
};

template <class T> class V2PScopedPtr<T[]> {
private:
	T *m_ptr;

public:
	V2PScopedPtr(T *ptr)
	    : m_ptr(ptr)
	{
	}

private:
	V2PScopedPtr(const V2PScopedPtr &);
	V2PScopedPtr &operator=(const V2PScopedPtr &);

public:
	~V2PScopedPtr() { delete[] m_ptr; }

	T *moveFrom()
	{
		T *ret = m_ptr;
		m_ptr = NULL;
		return ret;
	}

	T *get() const throw() { return m_ptr; }

	T &operator*() const throw() { return *m_ptr; }

	T &operator[](size_t index) const throw() { return m_ptr[index]; }
};

#endif // VIDEO2P_SCOPED_PTR_HPP
