/*
    Video2P -- video hosting engine for I2P network.
    Copyright (C) 2023 - 2025 Video2P Team

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <string.h>

#include "string.hpp"

V2PString::V2PString() throw()
    : m_str(0)
    , m_size(0)
{
}

V2PString::V2PString(const char *str) throw(V2PAllocatorException)
    : m_size(strlen(str))
{
	m_str = m_alloc(m_size + 1);
	strcpy(m_str, str);
}

V2PString::V2PString(const V2PString &other) throw(V2PAllocatorException)
    : m_size(other.m_size)
{
	m_str = m_alloc(m_size + 1);
	strcpy(m_str, other.m_str);
}

V2PString &
V2PString::operator=(const V2PString &other) throw(V2PAllocatorException)
{
	V2PAllocator::free(m_str);
	m_size = other.m_size;
	m_str = m_alloc(m_size + 1);
	strcpy(m_str, other.m_str);
	return *this;
}

V2PString::~V2PString() throw()
{
	V2PAllocator::free(m_str);
}

const char *
V2PString::get() const throw()
{
	return m_str;
}

size_t
V2PString::getSize() const throw()
{
	return m_size;
}

char *
V2PString::m_alloc(size_t size) throw(V2PAllocatorException)
{
	return reinterpret_cast<char *>(V2PAllocator::malloc(size + 1));
}
